@@ -31,11 +31,13 @@ from oslo_utils import excutils
 from oslo_utils import importutils
 import six
 from six import moves
+from stevedore import driver
 
 from oslo_messaging._drivers import base
 from oslo_messaging._drivers import common as rpc_common
 from oslo_messaging._executors import base as executor_base  # FIXME(markmc)
-from oslo_messaging._i18n import _, _LE
+from oslo_messaging._i18n import _, _LE, _LW
+from oslo_messaging._drivers import pool
 
 
 zmq = importutils.try_import('eventlet.green.zmq')
@@ -56,8 +58,7 @@ zmq_opts = [
     # The module.Class to use for matchmaking.
     cfg.StrOpt(
         'rpc_zmq_matchmaker',
-        default=('oslo_messaging._drivers.'
-                 'matchmaker.MatchMakerLocalhost'),
+        default='local',
         help='MatchMaker driver.',
     ),
 
@@ -106,7 +107,7 @@ def _serialize(data):
 
 def _deserialize(data):
     """Deserialization wrapper."""
-    LOG.debug("Deserializing: %s", data)
+    LOG.debug("Deserializing: %r", data)
     return jsonutils.loads(data)
 
 
@@ -117,8 +118,8 @@ class ZmqSocket(object):
     Can be used as a Context (supports the 'with' statement).
     """
 
-    def __init__(self, addr, zmq_type, bind=True, subscribe=None):
-        self.ctxt = zmq.Context(CONF.rpc_zmq_contexts)
+    def __init__(self, addr, zmq_type, bind=True, subscribe=None, ctxt=None):
+        self.ctxt = ctxt or zmq.Context(CONF.rpc_zmq_contexts)
         self.sock = self.ctxt.socket(zmq_type)
 
         # Enable IPv6-support in libzmq.
@@ -180,7 +181,10 @@ class ZmqSocket(object):
         LOG.debug("Subscribing to %s", msg_filter)
 
         try:
-            self.sock.setsockopt(zmq.SUBSCRIBE, msg_filter)
+            arg = msg_filter
+            if six.PY3:
+                arg = arg.encode('utf-8')
+            self.sock.setsockopt(zmq.SUBSCRIBE, arg)
         except Exception:
             return
 
@@ -190,7 +194,10 @@ class ZmqSocket(object):
         """Unsubscribe."""
         if msg_filter not in self.subscriptions:
             return
-        self.sock.setsockopt(zmq.UNSUBSCRIBE, msg_filter)
+        arg = msg_filter
+        if six.PY3:
+            arg = arg.encode('utf-8')
+        self.sock.setsockopt(zmq.UNSUBSCRIBE, arg)
         self.subscriptions.remove(msg_filter)
 
     @property
@@ -236,11 +243,15 @@ class ZmqSocket(object):
 class ZmqClient(object):
     """Client for ZMQ sockets."""
 
-    def __init__(self, addr):
-        self.outq = ZmqSocket(addr, zmq.PUSH, bind=False)
+    def __init__(self, addr, ctxt=None):
+        self.address = addr
+        self.outq = ZmqSocket(addr, zmq.PUSH, bind=False, ctxt=ctxt)
 
     def cast(self, msg_id, topic, data, envelope):
-        msg_id = msg_id or 0
+        msg_id = msg_id or '0'
+
+        if six.PY3:
+            msg_id = msg_id.encode('utf-8')
 
         if not envelope:
             data = _serialize(data)
@@ -259,6 +270,67 @@ class ZmqClient(object):
         self.outq.close()
 
 
+class ZmqClientContext(object):
+    """This is essentially a wrapper around ZmqClient that supports 'with'.
+    It can also return a new ZmqClient, or one from a pool.
+
+    The function will also catch when an instance of this class is to be
+    deleted.  With that we can return ZmqClients to the pool on exceptions
+    and so forth without making the caller be responsible for catching them.
+    If possible the function makes sure to return a client to the pool.
+
+    Based on amqp.ConnectionContext.
+    """
+
+    def __init__(self, address, connection_pool=None, pooled=False):
+        self.connection = None
+        self.connection_pool = connection_pool
+        self.pooled = pooled
+        if self.pooled and self.connection_pool is not None:
+            self.connection = self.connection_pool.get(address)
+        else:
+            self.connection = ZmqClient(address)
+
+    def __enter__(self):
+        """When with ZmqClientContext() is used, return self."""
+        return self
+
+    def _done(self):
+        """If the client came from a pool, clean it up and put it back.
+        If it did not come from a pool, close it.
+        """
+        if self.connection:
+            if self.pooled and self.connection_pool is not None:
+                # Reset the connection so it's ready for the next caller
+                # to grab from the pool
+                self.connection_pool.put(self.connection)
+            else:
+                try:
+                    self.connection.close()
+                except Exception:
+                    pass
+            self.connection = None
+
+    def __exit__(self, exc_type, exc_value, tb):
+        """End of 'with' statement.  We're done here."""
+        self._done()
+
+    def __del__(self):
+        """Caller is done with this client.  Make sure we cleaned up."""
+        self._done()
+
+    def close(self):
+        """Caller is done with this client."""
+        self._done()
+
+    def __getattr__(self, key):
+        """Proxy all other calls to the ZmqClient instance."""
+        if self.connection:
+            return getattr(self.connection, key)
+        else:
+            raise rpc_common.InvalidRPCConnectionReuse()
+
+
 class RpcContext(rpc_common.CommonRpcContext):
     """Context that supports replying to a rpc.call."""
     def __init__(self, **kwargs):
@@ -320,7 +392,7 @@ class InternalContext(object):
             return {'exc':
                     rpc_common.serialize_remote_exception(sys.exc_info())}
 
-    def reply(self, ctx, proxy,
+    def reply(self, driver, ctx, proxy,
               msg_id=None, context=None, topic=None, msg=None):
         """Reply to a casted call."""
         # NOTE(ewindisch): context kwarg exists for Grizzly compat.
@@ -336,19 +408,20 @@ class InternalContext(object):
             ctx.replies)
 
         LOG.debug("Sending reply")
-        _multi_send(_cast, ctx, topic, {
+        _multi_send(driver, _cast, ctx, topic, {
             'method': '-process_reply',
             'args': {
                 'msg_id': msg_id,  # Include for Folsom compat.
                 'response': response
             }
-        }, _msg_id=msg_id)
+        }, _msg_id=msg_id, pooled=True)
 
 
 class ConsumerBase(object):
     """Base Consumer."""
 
-    def __init__(self):
+    def __init__(self, driver):
+        self.driver = driver
         self.private_ctx = InternalContext(None)
 
     @classmethod
@@ -371,7 +444,7 @@ class ConsumerBase(object):
         # Internal method
         # uses internal context for safety.
         if method == '-reply':
-            self.private_ctx.reply(ctx, proxy, **data['args'])
+            self.private_ctx.reply(self.driver, ctx, proxy, **data['args'])
             return
 
         proxy.dispatch(ctx, data)
@@ -383,9 +456,10 @@ class ZmqBaseReactor(ConsumerBase):
     Used for RoundRobin requests.
     """
 
-    def __init__(self, conf):
-        super(ZmqBaseReactor, self).__init__()
+    def __init__(self, conf, driver=None):
+        super(ZmqBaseReactor, self).__init__(driver)
 
+        self.driver = driver
         self.proxies = {}
         self.threads = []
         self.sockets = []
@@ -564,8 +638,8 @@ class ZmqReactor(ZmqBaseReactor):
     Can also be used as a 1:1 proxy
     """
 
-    def __init__(self, conf):
-        super(ZmqReactor, self).__init__(conf)
+    def __init__(self, conf, driver):
+        super(ZmqReactor, self).__init__(conf, driver)
 
     def consume(self, sock):
         # TODO(ewindisch): use zero-copy (i.e. references, not copying)
@@ -574,13 +648,13 @@ class ZmqReactor(ZmqBaseReactor):
 
         proxy = self.proxies[sock]
 
-        if data[2] == 'cast':  # Legacy protocol
+        if data[2] == b'cast':  # Legacy protocol
             packenv = data[3]
 
             ctx, msg = _deserialize(packenv)
             request = rpc_common.deserialize_msg(msg)
             ctx = RpcContext.unmarshal(ctx)
-        elif data[2] == 'impl_zmq_v2':
+        elif data[2] == b'impl_zmq_v2':
             packenv = data[4:]
 
             msg = unflatten_envelope(packenv)
@@ -598,9 +672,9 @@ class ZmqReactor(ZmqBaseReactor):
 class Connection(rpc_common.Connection):
     """Manages connections and threads."""
 
-    def __init__(self, conf):
+    def __init__(self, conf, driver):
         self.topics = []
-        self.reactor = ZmqReactor(conf)
+        self.reactor = ZmqReactor(conf, driver)
 
     def create_consumer(self, topic, proxy, fanout=False):
         # Register with matchmaker.
@@ -653,8 +727,8 @@ class Connection(rpc_common.Connection):
         self.reactor.consume_in_thread()
 
 
-def _cast(addr, context, topic, msg, timeout=None, envelope=False,
-          _msg_id=None, allowed_remote_exmods=None):
+def _cast(driver, addr, context, topic, msg, timeout=None, envelope=False,
+          _msg_id=None, allowed_remote_exmods=None, pooled=False):
     allowed_remote_exmods = allowed_remote_exmods or []
     timeout_cast = timeout or CONF.rpc_cast_timeout
     payload = [RpcContext.marshal(context), msg]
@@ -662,21 +736,16 @@ def _cast(addr, context, topic, msg, timeout=None, envelope=False,
         topic = topic.encode('utf-8')
 
     with Timeout(timeout_cast, exception=rpc_common.Timeout):
-        conn = None
-        try:
-            conn = ZmqClient(addr)
-
-            # assumes cast can't return an exception
-            conn.cast(_msg_id, topic, payload, envelope)
-        except zmq.ZMQError:
-            raise RPCException("Cast failed. ZMQ Socket Exception")
-        finally:
-            if conn is not None:
-                conn.close()
+        with driver.get_connection(addr, pooled) as conn:
+            try:
+                # assumes cast can't return an exception
+                conn.cast(_msg_id, topic, payload, envelope)
+            except zmq.ZMQError:
+                raise RPCException("Cast failed. ZMQ Socket Exception")
 
 
-def _call(addr, context, topic, msg, timeout=None,
-          envelope=False, allowed_remote_exmods=None):
+def _call(driver, addr, context, topic, msg, timeout=None,
+          envelope=False, allowed_remote_exmods=None, pooled=False):
     allowed_remote_exmods = allowed_remote_exmods or []
     # timeout_response is how long we wait for a response
     timeout = timeout or CONF.rpc_response_timeout
@@ -714,7 +783,8 @@ def _call(addr, context, topic, msg, timeout=None,
             )
 
             LOG.debug("Sending cast: %s", topic)
-            _cast(addr, context, topic, payload, envelope=envelope)
+            _cast(driver, addr, context, topic, payload, envelope=envelope,
+                  pooled=pooled)
 
             LOG.debug("Cast sent; Waiting reply")
             # Blocks until receives reply
@@ -724,9 +794,9 @@ def _call(addr, context, topic, msg, timeout=None,
             LOG.debug("Received message: %s", msg)
             LOG.debug("Unpacking response")
 
-            if msg[2] == 'cast':  # Legacy version
+            if msg[2] == b'cast':  # Legacy version
                 raw_msg = _deserialize(msg[-1])[-1]
-            elif msg[2] == 'impl_zmq_v2':
+            elif msg[2] == b'impl_zmq_v2':
                 rpc_envelope = unflatten_envelope(msg[4:])
                 raw_msg = rpc_common.deserialize_msg(rpc_envelope)
             else:
@@ -748,15 +818,16 @@ def _call(addr, context, topic, msg, timeout=None,
     # One effect of this is that we're checking all
     # responses for Exceptions.
     for resp in responses:
-        if isinstance(resp, types.DictType) and 'exc' in resp:
+        if isinstance(resp, dict) and 'exc' in resp:
             raise rpc_common.deserialize_remote_exception(
                 resp['exc'], allowed_remote_exmods)
 
     return responses[-1]
 
 
-def _multi_send(method, context, topic, msg, timeout=None,
-                envelope=False, _msg_id=None, allowed_remote_exmods=None):
+def _multi_send(driver, method, context, topic, msg, timeout=None,
+                envelope=False, _msg_id=None, allowed_remote_exmods=None,
+                pooled=False):
     """Wraps the sending of messages.
 
     Dispatches to the matchmaker and sends message to all relevant hosts.
@@ -770,10 +841,15 @@ def _multi_send(method, context, topic, msg, timeout=None,
 
     # Don't stack if we have no matchmaker results
     if not queues:
-        LOG.warn(_("No matchmaker results. Not casting."))
+        warn_log = _LW("No matchmaker results. Not sending.")
+
+        if method.__name__ == '_cast':
+            LOG.warn(warn_log)
+            return
+
         # While not strictly a timeout, callers know how to handle
         # this exception and a timeout isn't too big a lie.
-        raise rpc_common.Timeout(_("No match from matchmaker."))
+        raise rpc_common.Timeout(warn_log)
 
     # This supports brokerless fanout (addresses > 1)
     return_val = None
@@ -782,25 +858,38 @@ def _multi_send(method, context, topic, msg, timeout=None,
         _addr = "tcp://%s:%s" % (ip_addr, conf.rpc_zmq_port)
 
         if method.__name__ == '_cast':
-            eventlet.spawn_n(method, _addr, context,
-                             _topic, msg, timeout, envelope, _msg_id)
+            eventlet.spawn_n(method, driver, _addr, context,
+                             _topic, msg, timeout, envelope, _msg_id,
+                             None, pooled)
         else:
-            return_val = method(_addr, context, _topic, msg, timeout,
-                                envelope, allowed_remote_exmods)
+            return_val = method(driver, _addr, context, _topic, msg, timeout,
+                                envelope, allowed_remote_exmods, pooled)
 
     return return_val
 
 
 def _get_matchmaker(*args, **kwargs):
     global matchmaker
+    mm_name = CONF.rpc_zmq_matchmaker
+
+    # Back compatibility for old class names
+    mm_mapping = {
+        'oslo_messaging._drivers.matchmaker_redis.MatchMakerRedis': 'redis',
+        'oslo_messaging._drivers.matchmaker_ring.MatchMakerRing': 'ring',
+        'oslo_messaging._drivers.matchmaker.MatchMakerLocalhost': 'local',
+        'oslo.messaging._drivers.matchmaker_redis.MatchMakerRedis': 'redis',
+        'oslo.messaging._drivers.matchmaker_ring.MatchMakerRing': 'ring',
+        'oslo.messaging._drivers.matchmaker.MatchMakerLocalhost': 'local'}
+    if mm_name in mm_mapping:
+        LOG.warn(_LW('rpc_zmq_matchmaker = %(old_val)s is deprecated. '
+                     'It is suggested to change the value to %(new_val)s.'),
+                 {'old_val': mm_name, 'new_val': mm_mapping[mm_name]})
+        mm_name = mm_mapping[mm_name]
+
     if not matchmaker:
-        mm = CONF.rpc_zmq_matchmaker
-        if mm.endswith('matchmaker.MatchMakerRing'):
-            mm.replace('matchmaker', 'matchmaker_ring')
-            LOG.warn(_('rpc_zmq_matchmaker = %(orig)s is deprecated; use'
-                       ' %(new)s instead') % dict(
-                     orig=CONF.rpc_zmq_matchmaker, new=mm))
-        matchmaker = importutils.import_object(mm, *args, **kwargs)
+        mgr = driver.DriverManager('oslo.messaging.zmq.matchmaker',
+                                   mm_name)
+        matchmaker = mgr.driver(*args, **kwargs)
     return matchmaker
 
 
@@ -854,7 +943,56 @@ class ZmqListener(base.Listener):
             return None
 
 
+class ZmqClientPool(pool.Pool):
+    """Class that implements a pool of Zmq Clients for a single endpoint"""
+    def __init__(self, conf, address, connection_cls, ctxt):
+        self.connection_cls = connection_cls
+        self.ctxt = ctxt
+        self.address = address
+        super(ZmqClientPool, self).__init__(conf.rpc_conn_pool_size)
+
+    def create(self):
+        LOG.debug('Pool creating new ZMQ connection for %s' % self.address)
+        return self.connection_cls(self.address, self.ctxt)
+
+    def empty(self):
+        for item in self.iter_free():
+            item.close()
+
+
+class ZmqClientPoolManager(object):
+    """Class that manages pools of clients for Zmq endpoints"""
+
+    def __init__(self, conf, ctxt=None):
+        self._pools = {}
+        self._lock = threading.Lock()
+        self.conf = conf
+        self.ctxt = ctxt
+
+    def get(self, address):
+        if address not in self._pools:
+            with self._lock:
+                if address not in self._pools:
+                    self._pools[address] = ZmqClientPool(self.conf,
+                                                         address,
+                                                         ZmqClient,
+                                                         self.ctxt)
+        return self._pools[address].get()
+
+    def put(self, item):
+        self._pools[item.address].put(item)
+
+    def empty(self):
+        for p in self._pools:
+            self._pools[p].empty()
+
+
 class ZmqDriver(base.BaseDriver):
+    """ZeroMQ Driver
+
+    See :doc:`zmq_driver` for details.
+
+    """
 
     # FIXME(markmc): allow this driver to be used without eventlet
 
@@ -864,6 +1002,7 @@ class ZmqDriver(base.BaseDriver):
             raise ImportError("Failed to import eventlet.green.zmq")
         conf.register_opts(zmq_opts)
         conf.register_opts(executor_base._pool_opts)
+        conf.register_opts(base.base_opts)
 
         super(ZmqDriver, self).__init__(conf, url, default_exchange,
                                         allowed_remote_exmods)
@@ -882,6 +1021,33 @@ class ZmqDriver(base.BaseDriver):
 
         self.listeners = []
 
+        # NOTE(jamespage): Create pool manager on first use to deal with
+        #                  os.fork calls in openstack daemons.
+        self._pool = None
+        self._pid = None
+        self._lock = threading.Lock()
+
+    def _configure_pool_manager(func):
+        """Causes a new pool manager to be created when the messaging service
+        is first used by the current process.  This is important as all
+        connections in the pools manager by the pool manager will share the
+        same ZMQ context, which must not be shared across OS processes.
+        """
+        def wrap(self, *args, **kws):
+            with self._lock:
+                old_pid = self._pid
+                self._pid = os.getpid()
+
+            if old_pid != self._pid:
+                # Create fresh pool manager for the current process
+                # along with a new ZMQ context.
+                self._pool = ZmqClientPoolManager(
+                    self.conf,
+                    zmq.Context(self.conf.rpc_zmq_contexts)
+                )
+            return func(self, *args, **kws)
+        return wrap
+
     def _send(self, target, ctxt, message,
               wait_for_reply=None, timeout=None, envelope=False):
 
@@ -898,19 +1064,22 @@ class ZmqDriver(base.BaseDriver):
         elif target.server:
             topic = '%s.%s' % (topic, target.server)
 
-        reply = _multi_send(method, ctxt, topic, message,
+        reply = _multi_send(self, method, ctxt, topic, message,
                             envelope=envelope,
-                            allowed_remote_exmods=self._allowed_remote_exmods)
+                            allowed_remote_exmods=self._allowed_remote_exmods,
+                            pooled=True)
 
         if wait_for_reply:
             return reply[-1]
 
+    @_configure_pool_manager
     def send(self, target, ctxt, message, wait_for_reply=None, timeout=None,
              retry=None):
         # NOTE(sileht): retry is not implemented because this driver never
         # retry anything
         return self._send(target, ctxt, message, wait_for_reply, timeout)
 
+    @_configure_pool_manager
     def send_notification(self, target, ctxt, message, version, retry=None):
         # NOTE(ewindisch): dot-priority in rpc notifier does not
         # work with our assumptions.
@@ -919,8 +1088,9 @@ class ZmqDriver(base.BaseDriver):
         target = target(topic=target.topic.replace('.', '-'))
         return self._send(target, ctxt, message, envelope=(version == 2.0))
 
+    @_configure_pool_manager
     def listen(self, target):
-        conn = Connection(self.conf)
+        conn = Connection(self.conf, self)
 
         listener = ZmqListener(self)
 
@@ -934,12 +1104,13 @@ class ZmqDriver(base.BaseDriver):
 
         return listener
 
+    @_configure_pool_manager
     def listen_for_notifications(self, targets_and_priorities, pool):
         # NOTE(sileht): this listener implementation is limited
         # because zeromq doesn't support:
         #  * requeing message
         #  * pool
-        conn = Connection(self.conf)
+        conn = Connection(self.conf, self)
 
         listener = ZmqListener(self)
         for target, priority in targets_and_priorities:
@@ -957,3 +1128,8 @@ class ZmqDriver(base.BaseDriver):
         for c in self.listeners:
             c.close()
         self.listeners = []
+        if self._pool:
+            self._pool.empty()
+
+    def get_connection(self, address, pooled=False):
+        return ZmqClientContext(address, self._pool, pooled)
