@@ -22,7 +22,6 @@ import uuid
 import fixtures
 import kombu
 import kombu.transport.memory
-import mock
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from oslotest import mockpatch
@@ -34,6 +33,7 @@ from oslo_messaging._drivers import amqpdriver
 from oslo_messaging._drivers import common as driver_common
 from oslo_messaging._drivers import impl_rabbit as rabbit_driver
 from oslo_messaging.tests import utils as test_utils
+from six.moves import mock
 
 load_tests = testscenarios.load_tests_apply_scenarios
 
@@ -58,6 +58,12 @@ class TestDeprecatedRabbitDriverLoad(test_utils.BaseTestCase):
 
 class TestHeartbeat(test_utils.BaseTestCase):
 
+    def setUp(self):
+        super(TestHeartbeat, self).setUp(
+            conf=cfg.ConfigOpts())
+        self.config(heartbeat_timeout_threshold=60,
+                    group='oslo_messaging_rabbit')
+
     @mock.patch('oslo_messaging._drivers.impl_rabbit.LOG')
     @mock.patch('kombu.connection.Connection.heartbeat_check')
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.'
@@ -163,44 +169,109 @@ class TestRabbitDriverLoadSSL(test_utils.BaseTestCase):
 
         transport._driver._get_connection()
         connection_klass.assert_called_once_with(
-            'memory:///', ssl=self.expected, login_method='AMQPLAIN',
-            heartbeat=60, failover_strategy="shuffle")
+            'memory:///', transport_options={'confirm_publish': True,
+                                             'on_blocked': mock.ANY,
+                                             'on_unblocked': mock.ANY},
+            ssl=self.expected, login_method='AMQPLAIN',
+            heartbeat=0, failover_strategy="shuffle")
 
 
-class TestRabbitIterconsume(test_utils.BaseTestCase):
+class TestRabbitPublisher(test_utils.BaseTestCase):
+    @mock.patch('kombu.messaging.Producer.publish')
+    def test_send_with_timeout(self, fake_publish):
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        with transport._driver._get_connection(amqp.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            conn._publish(mock.Mock(), 'msg', routing_key='routing_key',
+                          timeout=1)
+        fake_publish.assert_called_with('msg', expiration=1000)
 
-    def test_iterconsume_timeout(self):
+    @mock.patch('kombu.messaging.Producer.publish')
+    def test_send_no_timeout(self, fake_publish):
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        with transport._driver._get_connection(amqp.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            conn._publish(mock.Mock(), 'msg', routing_key='routing_key')
+        fake_publish.assert_called_with('msg', expiration=None)
+
+    def test_declared_queue_publisher(self):
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
         self.addCleanup(transport.cleanup)
-        deadline = time.time() + 3
+
+        e_passive = kombu.entity.Exchange(
+            name='foobar',
+            type='topic',
+            passive=True)
+
+        e_active = kombu.entity.Exchange(
+            name='foobar',
+            type='topic',
+            passive=False)
+
+        with transport._driver._get_connection(amqp.PURPOSE_SEND) as pool_conn:
+            conn = pool_conn.connection
+            exc = conn.connection.channel_errors[0]
+
+            def try_send(exchange):
+                conn._ensure_publishing(
+                    conn._publish_and_creates_default_queue,
+                    exchange, {}, routing_key='foobar')
+
+            # Ensure the exchange does not exists
+            self.assertRaises(exc, try_send, e_passive)
+            # Create it
+            try_send(e_active)
+            # Ensure it creates it
+            try_send(e_passive)
+
+            with mock.patch('kombu.messaging.Producer', side_effect=exc):
+                # Shoud reset the cache and ensures the exchange does
+                # not exists
+                self.assertRaises(exc, try_send, e_passive)
+            # Recreate it
+            try_send(e_active)
+            # Ensure it have been recreated
+            try_send(e_passive)
+
+
+class TestRabbitConsume(test_utils.BaseTestCase):
+
+    def test_consume_timeout(self):
+        transport = oslo_messaging.get_transport(self.conf,
+                                                 'kombu+memory:////')
+        self.addCleanup(transport.cleanup)
+        deadline = time.time() + 6
         with transport._driver._get_connection(amqp.PURPOSE_LISTEN) as conn:
-            conn.iterconsume(timeout=3)
+            self.assertRaises(driver_common.Timeout,
+                              conn.consume, timeout=3)
+
             # kombu memory transport doesn't really raise error
             # so just simulate a real driver behavior
             conn.connection.connection.recoverable_channel_errors = (IOError,)
             conn.declare_fanout_consumer("notif.info", lambda msg: True)
             with mock.patch('kombu.connection.Connection.drain_events',
                             side_effect=IOError):
-                try:
-                    conn.consume(timeout=3)
-                except driver_common.Timeout:
-                    pass
+                self.assertRaises(driver_common.Timeout,
+                                  conn.consume, timeout=3)
 
         self.assertEqual(0, int(deadline - time.time()))
 
-    def test_connection_reset_always_succeed(self):
+    def test_connection_ack_have_disconnected_kombu_connection(self):
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
         self.addCleanup(transport.cleanup)
-        channel = mock.Mock()
-        conn = transport._driver._get_connection(amqp.PURPOSE_LISTEN
-                                                 ).connection
-        conn.connection.recoverable_channel_errors = (IOError,)
-        with mock.patch.object(conn.connection, 'channel',
-                               side_effect=[IOError, IOError, channel]):
-            conn.reset()
-            self.assertEqual(channel, conn.channel)
+        with transport._driver._get_connection(amqp.PURPOSE_LISTEN) as conn:
+            channel = conn.connection.channel
+            with mock.patch('kombu.connection.Connection.connected',
+                            new_callable=mock.PropertyMock,
+                            return_value=False):
+                self.assertRaises(driver_common.Timeout,
+                                  conn.connection.consume, timeout=0.01)
+                # Ensure a new channel have been setuped
+                self.assertNotEqual(channel, conn.connection.channel)
 
 
 class TestRabbitTransportURL(test_utils.BaseTestCase):
@@ -234,6 +305,13 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
               expected=["amqp://user:password@host:10/virtual_host",
                         "amqp://user2:password2@host2:12/virtual_host"]
               )),
+        ('rabbit_ipv6',
+         dict(url='rabbit://u:p@[fd00:beef:dead:55::133]:10/vhost',
+              skip_py26='python 2.6 has broken urlparse for ipv6',
+              expected=['amqp://u:p@[fd00:beef:dead:55::133]:10/vhost'])),
+        ('rabbit_ipv4',
+         dict(url='rabbit://user:password@10.20.30.40:10/vhost',
+              expected=['amqp://user:password@10.20.30.40:10/vhost'])),
     ]
 
     def setUp(self):
@@ -245,6 +323,9 @@ class TestRabbitTransportURL(test_utils.BaseTestCase):
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.ensure')
     @mock.patch('oslo_messaging._drivers.impl_rabbit.Connection.reset')
     def test_transport_url(self, fake_reset, fake_ensure):
+        if hasattr(self, 'skip_py26') and sys.version_info < (2, 7):
+            self.skipTest(self.skip_py26)
+
         transport = oslo_messaging.get_transport(self.conf, self.url)
         self.addCleanup(transport.cleanup)
         driver = transport._driver
@@ -285,17 +366,25 @@ class TestSendReceive(test_utils.BaseTestCase):
         ('timeout', dict(timeout=0.01)),  # FIXME(markmc): timeout=0 is broken?
     ]
 
+    _reply_ending = [
+        ('old_behavior', dict(send_single_reply=False)),
+        ('new_behavior', dict(send_single_reply=True)),
+    ]
+
     @classmethod
     def generate_scenarios(cls):
         cls.scenarios = testscenarios.multiply_scenarios(cls._n_senders,
                                                          cls._context,
                                                          cls._reply,
                                                          cls._failure,
-                                                         cls._timeout)
+                                                         cls._timeout,
+                                                         cls._reply_ending)
 
     def test_send_receive(self):
         self.config(heartbeat_timeout_threshold=0,
                     group="oslo_messaging_rabbit")
+        self.config(send_single_reply=self.send_single_reply,
+                    group="oslo_messaging_rabbit")
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
         self.addCleanup(transport.cleanup)
@@ -520,7 +609,7 @@ def _declare_queue(target):
                                    channel=channel,
                                    exchange=exchange,
                                    routing_key=target.topic)
-    if target.server:
+    elif target.server:
         exchange = kombu.entity.Exchange(name='openstack',
                                          type='topic',
                                          durable=False,
@@ -552,10 +641,8 @@ class TestRequestWireFormat(test_utils.BaseTestCase):
          dict(topic='testtopic', server=None, fanout=False)),
         ('server_target',
          dict(topic='testtopic', server='testserver', fanout=False)),
-        # NOTE(markmc): https://github.com/celery/kombu/issues/195
         ('fanout_target',
-         dict(topic='testtopic', server=None, fanout=True,
-              skip_msg='Requires kombu>2.5.12 to fix kombu issue #195')),
+         dict(topic='testtopic', server=None, fanout=True)),
     ]
 
     _msg = [
@@ -593,8 +680,6 @@ class TestRequestWireFormat(test_utils.BaseTestCase):
         return self.uuids[-1]
 
     def test_request_wire_format(self):
-        if hasattr(self, 'skip_msg'):
-            self.skipTest(self.skip_msg)
 
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
@@ -696,10 +781,8 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
          dict(topic='testtopic', server=None, fanout=False)),
         ('server_target',
          dict(topic='testtopic', server='testserver', fanout=False)),
-        # NOTE(markmc): https://github.com/celery/kombu/issues/195
         ('fanout_target',
-         dict(topic='testtopic', server=None, fanout=True,
-              skip_msg='Requires kombu>2.5.12 to fix kombu issue #195')),
+         dict(topic='testtopic', server=None, fanout=True)),
     ]
 
     _msg = [
@@ -727,8 +810,6 @@ class TestReplyWireFormat(test_utils.BaseTestCase):
                                                          cls._target)
 
     def test_reply_wire_format(self):
-        if hasattr(self, 'skip_msg'):
-            self.skipTest(self.skip_msg)
 
         transport = oslo_messaging.get_transport(self.conf,
                                                  'kombu+memory:////')
@@ -847,15 +928,15 @@ class ConnectionLockTestCase(test_utils.BaseTestCase):
         l = rabbit_driver.ConnectionLock()
         t1 = self._thread(l, 1)
         t2 = self._thread(l, 1)
-        self.assertAlmostEqual(1, t1(), places=1)
-        self.assertAlmostEqual(2, t2(), places=1)
+        self.assertAlmostEqual(1, t1(), places=0)
+        self.assertAlmostEqual(2, t2(), places=0)
 
     def test_worker_and_heartbeat(self):
         l = rabbit_driver.ConnectionLock()
         t1 = self._thread(l, 1)
         t2 = self._thread(l, 1, heartbeat=True)
-        self.assertAlmostEqual(1, t1(), places=1)
-        self.assertAlmostEqual(2, t2(), places=1)
+        self.assertAlmostEqual(1, t1(), places=0)
+        self.assertAlmostEqual(2, t2(), places=0)
 
     def test_workers_and_heartbeat(self):
         l = rabbit_driver.ConnectionLock()
@@ -864,15 +945,15 @@ class ConnectionLockTestCase(test_utils.BaseTestCase):
         t3 = self._thread(l, 1)
         t4 = self._thread(l, 1, heartbeat=True)
         t5 = self._thread(l, 1)
-        self.assertAlmostEqual(1, t1(), places=1)
-        self.assertAlmostEqual(2, t4(), places=1)
-        self.assertAlmostEqual(3, t2(), places=1)
-        self.assertAlmostEqual(4, t3(), places=1)
-        self.assertAlmostEqual(5, t5(), places=1)
+        self.assertAlmostEqual(1, t1(), places=0)
+        self.assertAlmostEqual(2, t4(), places=0)
+        self.assertAlmostEqual(3, t2(), places=0)
+        self.assertAlmostEqual(4, t3(), places=0)
+        self.assertAlmostEqual(5, t5(), places=0)
 
     def test_heartbeat(self):
         l = rabbit_driver.ConnectionLock()
         t1 = self._thread(l, 1, heartbeat=True)
         t2 = self._thread(l, 1)
-        self.assertAlmostEqual(1, t1(), places=1)
-        self.assertAlmostEqual(2, t2(), places=1)
+        self.assertAlmostEqual(1, t1(), places=0)
+        self.assertAlmostEqual(2, t2(), places=0)
