@@ -20,19 +20,18 @@ import threading
 import time
 import uuid
 
+import six
 from six import moves
 import testtools
 
 from oslo import messaging
-from oslo.utils import importutils
 from oslo_messaging.tests import utils as test_utils
 
-# TODO(kgiusti) Conditionally run these tests only if the necessary
-# dependencies are installed.  This should be removed once the proton libraries
-# are available in the base repos for all supported platforms.
-pyngus = importutils.try_import("pyngus")
-if pyngus:
+if six.PY2:
+    # NOTE(flaper87): pyngus currently doesn't support py34. It's
+    # on the works, though.
     from oslo_messaging._drivers.protocols.amqp import driver as amqp_driver
+    import pyngus
 
 
 LOG = logging.getLogger(__name__)
@@ -71,7 +70,7 @@ class _ListenerThread(threading.Thread):
         return msgs
 
 
-@testtools.skipUnless(pyngus, "proton modules not present")
+@testtools.skipUnless(six.PY2, "No Py3K support yet")
 class TestProtonDriverLoad(test_utils.BaseTestCase):
 
     def setUp(self):
@@ -86,7 +85,7 @@ class TestProtonDriverLoad(test_utils.BaseTestCase):
 
 class _AmqpBrokerTestCase(test_utils.BaseTestCase):
 
-    @testtools.skipUnless(pyngus, "proton modules not present")
+    @testtools.skipUnless(six.PY2, "No Py3K support yet")
     def setUp(self):
         super(_AmqpBrokerTestCase, self).setUp()
         self._broker = FakeBroker()
@@ -258,29 +257,44 @@ class TestAmqpNotification(_AmqpBrokerTestCase):
         notifications = [(messaging.Target(topic="topic-1"), 'info'),
                          (messaging.Target(topic="topic-1"), 'error'),
                          (messaging.Target(topic="topic-2"), 'debug')]
-        nl = driver.listen_for_notifications(notifications)
+        nl = driver.listen_for_notifications(notifications, None)
 
-        listener = _ListenerThread(nl, 3)
+        # send one for each support version:
+        msg_count = len(notifications) * 2
+        listener = _ListenerThread(nl, msg_count)
         targets = ['topic-1.info',
-                   'topic-1.bad',  # should be dropped
-                   'bad-topic.debug',  # should be dropped
-                   'topic-1.error', 'topic-2.debug']
+                   'topic-1.bad',  # will raise MessagingDeliveryFailure
+                   'bad-topic.debug',  # will raise MessagingDeliveryFailure
+                   'topic-1.error',
+                   'topic-2.debug']
+
+        excepted_targets = []
+        exception_count = 0
+        for version in (1.0, 2.0):
+            for t in targets:
+                try:
+                    driver.send_notification(messaging.Target(topic=t),
+                                             "context", {'target': t},
+                                             version)
+                except messaging.MessageDeliveryFailure:
+                    exception_count += 1
+                    excepted_targets.append(t)
 
-        for t in targets:
-            driver.send_notification(messaging.Target(topic=t),
-                                     "context", {'target': t},
-                                     1.0)
         listener.join(timeout=30)
         self.assertFalse(listener.isAlive())
         topics = [x.message.get('target') for x in listener.get_messages()]
-        self.assertTrue('topic-1.info' in topics)
-        self.assertTrue('topic-1.error' in topics)
-        self.assertTrue('topic-2.debug' in topics)
-        self.assertEqual(self._broker.dropped_count, 2)
+        self.assertEqual(len(topics), msg_count)
+        self.assertEqual(topics.count('topic-1.info'), 2)
+        self.assertEqual(topics.count('topic-1.error'), 2)
+        self.assertEqual(topics.count('topic-2.debug'), 2)
+        self.assertEqual(self._broker.dropped_count, 4)
+        self.assertEqual(exception_count, 4)
+        self.assertEqual(excepted_targets.count('topic-1.bad'), 2)
+        self.assertEqual(excepted_targets.count('bad-topic.debug'), 2)
         driver.cleanup()
 
 
-@testtools.skipUnless(pyngus, "proton modules not present")
+@testtools.skipUnless(six.PY2, "No Py3K support yet")
 class TestAuthentication(test_utils.BaseTestCase):
 
     def setUp(self):
@@ -334,7 +348,7 @@ class TestAuthentication(test_utils.BaseTestCase):
         driver.cleanup()
 
 
-@testtools.skipUnless(pyngus, "proton modules not present")
+@testtools.skipUnless(six.PY2, "No Py3K support yet")
 class TestFailover(test_utils.BaseTestCase):
 
     def setUp(self):
@@ -409,7 +423,7 @@ class TestFailover(test_utils.BaseTestCase):
 class FakeBroker(threading.Thread):
     """A test AMQP message 'broker'."""
 
-    if pyngus:
+    if six.PY2:
         class Connection(pyngus.ConnectionEventHandler):
             """A single AMQP connection."""
 
